@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2019-2022 Scoopta
+ *  Copyright (C) 2019-2024 Scoopta
  *  This file is part of Wofi
  *  Wofi is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -42,8 +42,10 @@
 #include <gdk/gdkwayland.h>
 
 #define PROTO_VERSION(v1, v2) (v1 < v2 ? v1 : v2)
+#define _UNUSED(x) (void)(x)
+#define CUSTOM_KEY_NUMBER 20
 
-static const char* terminals[] = {"kitty", "termite", "alacritty", "foot", "gnome-terminal", "weston-terminal"};
+static const char* terminals[] = {"kitty", "alacritty", "wezterm", "foot", "termite", "gnome-terminal", "weston-terminal"};
 
 enum location {
 	LOCATION_CENTER,
@@ -105,9 +107,14 @@ static pthread_t mode_thread;
 static bool has_joined_mode = false;
 static char* copy_exec = NULL;
 static char* pre_display_cmd = NULL;
+static bool pre_display_exec = false;
 static bool single_click = false;
+static GdkModifierType shift_mask = GDK_SHIFT_MASK;
+static GdkModifierType ctrl_mask = GDK_CONTROL_MASK;
+static GdkModifierType alt_mask = GDK_MOD1_MASK;
 
 static struct map* keys;
+static struct map* mods;
 
 static struct wl_display* wl = NULL;
 static struct wl_surface* wl_surface;
@@ -127,6 +134,8 @@ struct key_entry {
 	void (*action)(void);
 };
 
+static int custom_key_return_code = EXIT_SUCCESS;
+
 static void nop() {}
 
 static void add_interface(void* data, struct wl_registry* registry, uint32_t name, const char* interface, uint32_t version) {
@@ -260,6 +269,9 @@ static char* parse_images(WofiPropertyBox* box, const char* text, bool create_wi
 	map_put(mode_map, "img-base64-noscale", "true");
 	map_put(mode_map, "text", "true");
 
+	if(text == NULL) {
+		return NULL;
+	}
 	char* original = strdup(text);
 	char* mode1 = NULL;
 	char* mode2 = NULL;
@@ -404,29 +416,87 @@ static GtkWidget* create_label(char* mode, char* text, char* search_text, char*
 		char line[128]; // you'd think this caps the line's length to 128, but it's just a buffer which due to the nature of fgets() splits on lines
 		size_t size = 0;
 		// first, prepare cmd_labeltext to be each entry's actual comamand to run, aka replacing 'cat %s' to be 'cat filename'
-		if ((asprintf(&cmd_labeltext, pre_display_cmd, nodetext)) == -1) {
+		if(asprintf(&cmd_labeltext, pre_display_cmd, nodetext) == -1) {
 			fprintf(stderr, "error parsing pre_display_cmd to run\n");
-			exit(EXIT_FAILURE);
+			wofi_exit(EXIT_FAILURE);
 		}
 		// then, run the command
-		fp_labeltext = popen(cmd_labeltext, "r");
-		if (fp_labeltext == NULL) {
+		if(pre_display_exec) {
+			int fds[2];
+			if(pipe(fds) == -1) {
+				perror("pipe broken");
+				wofi_exit(1);
+			}
+			if(fork() == 0) {
+				close(fds[0]);
+				dup2(fds[1], STDOUT_FILENO);
+
+				char* cmd = strdup(pre_display_cmd);
+				char* space = strchr(cmd, ' ');
+				if(space != NULL) {
+					*space = 0;
+				}
+
+
+				size_t space_count = 0;
+				char* tmp_space = space;
+				for(; (tmp_space = strchr(tmp_space + 1, ' ')) != NULL; ++space_count);
+
+				char** args = malloc((sizeof(char*) * (space_count + 2)));
+				args[0] = cmd;
+				args[1] = space + 1;
+				args[space_count + 2] = NULL;
+
+				//> 0 is used because args[0] is the command
+				for(size_t count = space_count; count > 0; --count) {
+					char* arg = strrchr(space + 1, ' ');
+					args[count + 1] = arg + 1;
+					*arg = 0;
+				}
+
+				for(size_t count = 1; count <= space_count + 1; ++count) {
+					if(strstr(args[count], "%s") != NULL) {
+						if(asprintf(&args[count], args[count], nodetext) == -1) {
+							fprintf(stderr, "error parsing pre_display_cmd to run\n");
+							exit(EXIT_FAILURE);
+						}
+					}
+				}
+
+				execvp(cmd, args);
+				free(cmd);
+				free(args);
+				fprintf(stderr, "error executing '%s'\n", cmd_labeltext);
+				exit(1);
+			}
+			close(fds[1]);
+
+			fp_labeltext = fdopen(fds[0], "r");
+		} else {
+			fp_labeltext = popen(cmd_labeltext, "r");
+		}
+		if(fp_labeltext == NULL) {
 			fprintf(stderr, "error executing '%s'\n", cmd_labeltext);
-			exit(EXIT_FAILURE);
-		} else if (fgets(line, sizeof(line), fp_labeltext) != NULL) {
+			wofi_exit(EXIT_FAILURE);
+		} else if(fgets(line, sizeof(line), fp_labeltext) != NULL) {
 			// lastly, read the output of said command, and put it into the text variable to be used for the label widgets
 			// consider using 'printf %.10s as your --pre-display-cmd to limit a string to a determined width. 10 here is an example
 			size += strlen(line)+1; // we need place for the \0 of strcpy
 			text = (char *) realloc(text, size);
 			strcpy(text, line);
-			while (fgets(line, sizeof(line), fp_labeltext) != NULL) {
+			while(fgets(line, sizeof(line), fp_labeltext) != NULL) {
 				size += strlen(line);
 				text = (char *) realloc(text, size);
 				strncat(text, line, size);
 			}
 		}
 
-		pclose(fp_labeltext);
+		if(pre_display_exec) {
+			fclose(fp_labeltext);
+			while(waitpid(-1, NULL, WNOHANG) > 0);
+		} else {
+			pclose(fp_labeltext);
+		}
 	}
 
 	if(allow_images) {
@@ -584,6 +654,7 @@ static gboolean _insert_widget(gpointer data) {
 		gtk_expander_set_label_widget(GTK_EXPANDER(parent), box);
 
 		GtkWidget* exp_box = gtk_list_box_new();
+		gtk_widget_set_name(exp_box, "expander-box");
 		gtk_list_box_set_activate_on_single_click(GTK_LIST_BOX(exp_box), single_click);
 		g_signal_connect(exp_box, "row-activated", G_CALLBACK(activate_item), NULL);
 		gtk_container_add(GTK_CONTAINER(parent), exp_box);
@@ -922,7 +993,7 @@ void wofi_term_run(const char* cmd) {
 		execlp(terminals[count], terminals[count], "-e", cmd, NULL);
 	}
 	fprintf(stderr, "No terminal emulator found please set term in config or use --term\n");
-	exit(1);
+	wofi_exit(1);
 }
 
 static void flag_box(GtkBox* box, GtkStateFlags flags) {
@@ -1051,29 +1122,48 @@ static gint do_sort(GtkFlowBoxChild* child1, GtkFlowBoxChild* child2, gpointer d
 }
 
 static void select_first(void) {
-	GtkFlowBoxChild* child = gtk_flow_box_get_child_at_index(GTK_FLOW_BOX(inner_box), 1);
+	GtkFlowBoxChild* child = gtk_flow_box_get_child_at_index(GTK_FLOW_BOX(inner_box), 0);
 	gtk_widget_grab_focus(GTK_WIDGET(child));
 	gtk_flow_box_select_child(GTK_FLOW_BOX(inner_box), GTK_FLOW_BOX_CHILD(child));
 }
 
-static GdkModifierType get_mask_from_keyval(guint keyval) {
-	switch(keyval) {
-	case GDK_KEY_Shift_L:
-	case GDK_KEY_Shift_R:
+static GdkModifierType get_mask_from_keystate(guint state) {
+	if ((state & GDK_SHIFT_MASK) == GDK_SHIFT_MASK) {
 		return GDK_SHIFT_MASK;
-	case GDK_KEY_Control_L:
-	case GDK_KEY_Control_R:
+	}
+
+	if ((state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK) {
 		return GDK_CONTROL_MASK;
-	case GDK_KEY_Alt_L:
-	case GDK_KEY_Alt_R:
+	}
+
+	// Alt
+	if ((state & GDK_MOD1_MASK) == GDK_MOD1_MASK) {
 		return GDK_MOD1_MASK;
+	}
+
+	return 0;
+}
+
+static char* get_name_from_mask(GdkModifierType mask) {
+	switch(mask) {
+	case GDK_SHIFT_MASK:
+		return "Shift";
+	case GDK_CONTROL_MASK:
+		return "Ctrl";
+	case GDK_MOD1_MASK:
+		return "Alt";
 	default:
-		return 0;
+		return NULL;
 	}
 }
 
 static GdkModifierType get_mask_from_name(char* name) {
-	return get_mask_from_keyval(gdk_keyval_from_name(name));
+	GdkModifierType* mask = map_get(mods, name);
+	if (mask) {
+		return *mask;
+	}
+
+	return 0;
 }
 
 static void move_up(void) {
@@ -1114,12 +1204,21 @@ static void move_forward(void) {
 		select_first();
 		return;
 	}
+
 	gtk_widget_child_focus(window, GTK_DIR_TAB_FORWARD);
+
+	if(gtk_widget_has_focus(entry)) {
+		select_first();
+	}
 }
 
 static void move_backward(void) {
 	user_moved = true;
 	gtk_widget_child_focus(window, GTK_DIR_TAB_BACKWARD);
+
+	if(gtk_widget_has_focus(entry)) {
+		move_backward();
+	}
 }
 
 static void move_pgup(void) {
@@ -1137,7 +1236,7 @@ static void move_pgdn(void) {
 }
 
 static void do_exit(void) {
-	exit(1);
+	wofi_exit(1);
 }
 
 static void do_expand(void) {
@@ -1178,7 +1277,7 @@ static void do_copy(void) {
 			int fds[2];
 			if (pipe(fds) == -1) {
 				perror("pipe broken");
-				exit(EXIT_FAILURE);
+				wofi_exit(EXIT_FAILURE);
 			}
 			if(fork() == 0) {
 				close(fds[1]);
@@ -1200,7 +1299,51 @@ static void do_copy(void) {
 	}
 }
 
+static void on_exit_set_custom_key_return_code(void) {
+	fflush(stdout);
+	fflush(stderr);
+	_exit(custom_key_return_code);
+}
+
+void wofi_exit(int status) {
+	fflush(stdout);
+	fflush(stderr);
+
+	if(status == EXIT_SUCCESS) {
+		_exit(custom_key_return_code);
+	}
+
+	_exit(status);
+}
+
+static void do_custom_key(int custom_key_num) {
+	custom_key_return_code = custom_key_num + 10;
+}
+
+static void do_custom_key_0(void) {do_custom_key(0);}
+static void do_custom_key_1(void) {do_custom_key(1);}
+static void do_custom_key_2(void) {do_custom_key(2);}
+static void do_custom_key_3(void) {do_custom_key(3);}
+static void do_custom_key_4(void) {do_custom_key(4);}
+static void do_custom_key_5(void) {do_custom_key(5);}
+static void do_custom_key_6(void) {do_custom_key(6);}
+static void do_custom_key_7(void) {do_custom_key(7);}
+static void do_custom_key_8(void) {do_custom_key(8);}
+static void do_custom_key_9(void) {do_custom_key(9);}
+static void do_custom_key_10(void) {do_custom_key(10);}
+static void do_custom_key_11(void) {do_custom_key(11);}
+static void do_custom_key_12(void) {do_custom_key(12);}
+static void do_custom_key_13(void) {do_custom_key(13);}
+static void do_custom_key_14(void) {do_custom_key(14);}
+static void do_custom_key_15(void) {do_custom_key(15);}
+static void do_custom_key_16(void) {do_custom_key(16);}
+static void do_custom_key_17(void) {do_custom_key(17);}
+static void do_custom_key_18(void) {do_custom_key(18);}
+static void do_custom_key_19(void) {do_custom_key(19);}
+
 static bool do_key_action(GdkEvent* event, char* mod, void (*action)(void)) {
+	// GTK children focus gets all messed up if we don't first blow away any
+	// modifier keystate that's currently happening.
 	if(mod != NULL) {
 		GdkModifierType mask = get_mask_from_name(mod);
 		if((event->key.state & mask) == mask) {
@@ -1280,9 +1423,24 @@ static gboolean key_press(GtkWidget* widget, GdkEvent* event, gpointer data) {
 		return FALSE;
 	}
 
-
 	bool key_success = true;
-	struct key_entry* key_ent = map_get(keys, gdk_keyval_name(event->key.keyval));
+	struct key_entry* key_ent = NULL;
+	char* mod = NULL;
+
+	if (has_mod(event->key.state)) {
+		char* keyval = gdk_keyval_name(event->key.keyval);
+		GdkModifierType mask = get_mask_from_keystate(event->key.state);
+		mod = get_name_from_mask(mask);
+		size_t len = strlen(mod) + 1 + strlen(keyval) + 1;
+		char* mod_keyval = malloc(len);
+		strcpy(mod_keyval, mod);
+		strcat(mod_keyval, "-");
+		strcat(mod_keyval, keyval);
+		key_ent = map_get(keys, mod_keyval);
+		free(mod_keyval);
+	} else {
+		key_ent = map_get(keys, gdk_keyval_name(event->key.keyval));
+	}
 
 	if(key_ent != NULL && key_ent->action != NULL) {
 		key_success = do_key_action(event, key_ent->mod, key_ent->action);
@@ -1459,7 +1617,7 @@ static struct mode* add_mode(char* _mode) {
 
 			if(init == NULL) {
 				fprintf(stderr, "I would love to show %s but Idk what it is\n", _mode);
-				exit(1);
+				wofi_exit(1);
 			}
 		}
 	}
@@ -1486,7 +1644,7 @@ static void* start_mode_thread(void* data) {
 	return NULL;
 }
 
-static void parse_mods(char* key, void (*action)(void)) {
+static void add_key_entry(char* key, void (*action)(void)) {
 	char* tmp = strdup(key);
 	char* save_ptr;
 	char* str = strtok_r(tmp, ",", &save_ptr);
@@ -1495,22 +1653,20 @@ static void parse_mods(char* key, void (*action)(void)) {
 			break;
 		}
 		char* hyphen = strchr(str, '-');
-		char* mod;
+		char* mod = NULL;
+		GdkModifierType modifier = 0;
 		if(hyphen != NULL) {
-			*hyphen = 0;
-			guint key1 = gdk_keyval_from_name(str);
-			guint key2 = gdk_keyval_from_name(hyphen + 1);
-			if(get_mask_from_keyval(key1) != 0) {
+			*hyphen = '\0';
+			modifier = get_mask_from_name(str);
+			if (modifier) {
 				mod = str;
 				str = hyphen + 1;
-			} else if(get_mask_from_keyval(key2) != 0) {
-				mod = hyphen + 1;
 			} else {
-				fprintf(stderr, "Neither %s nor %s is a modifier, this is not supported\n", str, hyphen + 1);
-				mod = NULL;
+				fprintf(stderr,
+						"Could not parse %s as a modifier. Valid modifiers are "
+						"Ctrl, Shift, and Alt.\n",
+						str);
 			}
-		} else {
-			mod = NULL;
 		}
 		struct key_entry* entry = malloc(sizeof(struct key_entry));
 		if(mod == NULL) {
@@ -1519,7 +1675,20 @@ static void parse_mods(char* key, void (*action)(void)) {
 			entry->mod = strdup(mod);
 		}
 		entry->action = action;
-		map_put_void(keys, str, entry);
+		char* map_key = NULL;
+
+		if (mod) {
+			size_t len = strlen(mod) + 1 + strlen(str) + 1;
+			map_key = malloc(len);
+			strcpy(map_key, mod);
+			strcat(map_key, "-");
+			strcat(map_key, str);
+		} else {
+			map_key = strdup(str);
+		}
+
+		map_put_void(keys, map_key, entry);
+		free(map_key);
 	} while((str = strtok_r(NULL, ",", &save_ptr)) != NULL);
 	free(tmp);
 }
@@ -1572,12 +1741,24 @@ static gboolean do_percent_size(gpointer data) {
 	return G_SOURCE_CONTINUE;
 }
 
+static gboolean do_percent_size_first(gpointer data){
+	gdk_threads_add_timeout(50, do_percent_size, data);
+	do_percent_size(data);
+	return G_SOURCE_REMOVE;
+}
+
 void wofi_init(struct map* _config) {
 	config = _config;
 	char* width_str = config_get(config, "width", "50%");
 	char* height_str = config_get(config, "height", "40%");
 	width = strtol(width_str, NULL, 10);
 	height = strtol(height_str, NULL, 10);
+
+	if(width > UINT16_MAX || height > UINT16_MAX) {
+		fprintf(stderr, "Do you actually have a monitor big enough to see this O_o? Dimensions can be no larger than %ux%u\n", UINT16_MAX, UINT16_MAX);
+		wofi_exit(1);
+	}
+
 	x = map_get(config, "x");
 	y = map_get(config, "y");
 	bool normal_window = strcmp(config_get(config, "normal_window", "false"), "true") == 0;
@@ -1622,39 +1803,94 @@ void wofi_init(struct map* _config) {
 	char* layer = config_get(config, "layer", "top");
 	copy_exec = config_get(config, "copy_exec", "wl-copy");
 	pre_display_cmd = map_get(config, "pre_display_cmd");
+	pre_display_exec = strcmp(config_get(config, "pre_display_exec", "false"), "true") == 0;
 	single_click = strcmp(config_get(config, "single_click", "false"), "true") == 0;
 
 	keys = map_init_void();
+	mods = map_init_void();
 
+	map_put_void(mods, "Shift", &shift_mask);
+	map_put_void(mods, "Ctrl", &ctrl_mask);
+	map_put_void(mods, "Alt", &alt_mask);
 
+	// First pass maps the default key entries.
+	// Second pass maps any custom key entry configs in addition to (or
+	// replacing) defaults.
+	for (int i = 0; i < 2; i++) {
+		char* key_default;
 
-	char* key_up = config_get(config, "key_up", "Up");
-	char* key_down = config_get(config, "key_down", "Down");
-	char* key_left = config_get(config, "key_left", "Left");
-	char* key_right = config_get(config, "key_right", "Right");
-	char* key_forward = config_get(config, "key_forward", "Tab");
-	char* key_backward = config_get(config, "key_backward", "ISO_Left_Tab");
-	char* key_submit = config_get(config, "key_submit", "Return");
-	char* key_exit = config_get(config, "key_exit", "Escape");
-	char* key_pgup = config_get(config, "key_pgup", "Page_Up");
-	char* key_pgdn = config_get(config, "key_pgdn", "Page_Down");
-	char* key_expand = config_get(config, "key_expand", "");
-	char* key_hide_search = config_get(config, "key_hide_search", "");
-	char* key_copy = config_get(config, "key_copy", "Control_L-c");
+		key_default = "Up";
+		char* key_up = (i == 0) ? "Up" : config_get(config, "key_up", key_default);
+		key_default = "Down";
+		char* key_down = (i == 0) ? key_default : config_get(config, "key_down", key_default);
+		key_default = "Left";
+		char* key_left = (i == 0) ? key_default : config_get(config, "key_left", key_default);
+		key_default = "Right";
+		char* key_right = (i == 0) ? key_default : config_get(config, "key_right", key_default);
+		key_default = "Tab";
+		char* key_forward = (i == 0) ? key_default : config_get(config, "key_forward", key_default);
+		key_default = "Shift-ISO_Left_Tab";
+		char* key_backward = (i == 0) ? key_default : config_get(config, "key_backward", key_default);
+		key_default = "Return";
+		char* key_submit = (i == 0) ? key_default : config_get(config, "key_submit", key_default);
+		key_default = "Escape";
+		char* key_exit = (i == 0) ? key_default : config_get(config, "key_exit", key_default);
+		key_default = "Page_Up";
+		char* key_pgup = (i == 0) ? key_default : config_get(config, "key_pgup", key_default);
+		key_default = "Page_Down";
+		char* key_pgdn = (i == 0) ? key_default : config_get(config, "key_pgdn", key_default);
+		key_default = "";
+		char* key_expand = (i == 0) ? key_default: config_get(config, "key_expand", key_default);
+		key_default = "";
+		char* key_hide_search = (i == 0) ? key_default: config_get(config, "key_hide_search", key_default);
+		key_default = "Ctrl-c";
+		char* key_copy = (i == 0) ? key_default : config_get(config, "key_copy", key_default);
 
-	parse_mods(key_up, move_up);
-	parse_mods(key_down, move_down);
-	parse_mods(key_left, move_left);
-	parse_mods(key_right, move_right);
-	parse_mods(key_forward, move_forward);
-	parse_mods(key_backward, move_backward);
-	parse_mods(key_submit, NULL); //submit is a special case, when a NULL action is encountered submit is used instead
-	parse_mods(key_exit, do_exit);
-	parse_mods(key_pgup, move_pgup);
-	parse_mods(key_pgdn, move_pgdn);
-	parse_mods(key_expand, do_expand);
-	parse_mods(key_hide_search, do_hide_search);
-	parse_mods(key_copy, do_copy);
+		char* keys_custom[CUSTOM_KEY_NUMBER];
+		key_default = "";
+		for (uint8_t kc_index = 0; kc_index < (sizeof(keys_custom) / sizeof(char*)); kc_index++) {
+			char config_entry[15];
+			sprintf(config_entry, "key_custom_%d", kc_index);
+			keys_custom[kc_index] = (i == 0) ? key_default : config_get(config, config_entry, key_default);
+		}
+
+		add_key_entry(key_up, move_up);
+		add_key_entry(key_down, move_down);
+		add_key_entry(key_left, move_left);
+		add_key_entry(key_right, move_right);
+		add_key_entry(key_forward, move_forward);
+		add_key_entry(key_backward, move_backward);
+		add_key_entry(key_submit, NULL); //submit is a special case, when a NULL action is encountered submit is used instead
+		add_key_entry(key_exit, do_exit);
+		add_key_entry(key_pgup, move_pgup);
+		add_key_entry(key_pgdn, move_pgdn);
+		add_key_entry(key_expand, do_expand);
+		add_key_entry(key_hide_search, do_hide_search);
+		add_key_entry(key_copy, do_copy);
+
+		#define CUSTOM_KEY_FUNC(index) do_custom_key_##index
+
+		add_key_entry(keys_custom[0], CUSTOM_KEY_FUNC(0));
+		add_key_entry(keys_custom[1], CUSTOM_KEY_FUNC(1));
+		add_key_entry(keys_custom[2], CUSTOM_KEY_FUNC(2));
+		add_key_entry(keys_custom[3], CUSTOM_KEY_FUNC(3));
+		add_key_entry(keys_custom[4], CUSTOM_KEY_FUNC(4));
+		add_key_entry(keys_custom[5], CUSTOM_KEY_FUNC(5));
+		add_key_entry(keys_custom[6], CUSTOM_KEY_FUNC(6));
+		add_key_entry(keys_custom[7], CUSTOM_KEY_FUNC(7));
+		add_key_entry(keys_custom[8], CUSTOM_KEY_FUNC(8));
+		add_key_entry(keys_custom[9], CUSTOM_KEY_FUNC(9));
+		add_key_entry(keys_custom[10], CUSTOM_KEY_FUNC(10));
+		add_key_entry(keys_custom[11], CUSTOM_KEY_FUNC(11));
+		add_key_entry(keys_custom[12], CUSTOM_KEY_FUNC(12));
+		add_key_entry(keys_custom[13], CUSTOM_KEY_FUNC(13));
+		add_key_entry(keys_custom[14], CUSTOM_KEY_FUNC(14));
+		add_key_entry(keys_custom[15], CUSTOM_KEY_FUNC(15));
+		add_key_entry(keys_custom[16], CUSTOM_KEY_FUNC(16));
+		add_key_entry(keys_custom[17], CUSTOM_KEY_FUNC(17));
+		add_key_entry(keys_custom[18], CUSTOM_KEY_FUNC(18));
+		add_key_entry(keys_custom[19], CUSTOM_KEY_FUNC(19));
+	}
 
 	modes = map_init_void();
 
@@ -1676,7 +1912,7 @@ void wofi_init(struct map* _config) {
 
 		if(wl == NULL) {
 			fprintf(stderr, "Failed to connect to wayland compositor\n");
-			exit(1);
+			wofi_exit(1);
 		}
 
 		struct wl_registry* registry = wl_display_get_registry(wl);
@@ -1824,8 +2060,7 @@ void wofi_init(struct map* _config) {
 		static char* geo_str[2];
 		geo_str[0] = width_str;
 		geo_str[1] = height_str;
-		gdk_threads_add_timeout(50, do_percent_size, geo_str);
-		do_percent_size(geo_str);
+		gdk_threads_add_timeout(25, do_percent_size_first, geo_str);
 	}
 
 	wl_list_init(&mode_list);
@@ -1836,4 +2071,6 @@ void wofi_init(struct map* _config) {
 
 	gtk_window_set_title(GTK_WINDOW(window), prompt);
 	gtk_widget_show_all(window);
+
+	atexit(on_exit_set_custom_key_return_code);
 }
